"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const command_1 = require("./command");
const cli_engine_config_1 = require("cli-engine-config");
const cli_flags_1 = require("cli-flags");
const nock = require("nock");
class Command extends command_1.Command {
    async run() {
        cli_ux_1.cli.log('foo');
    }
}
Command.topic = 'foo';
Command.command = 'bar';
Command.flags = { myflag: cli_flags_1.flags.boolean() };
Command.args = [{ name: 'myarg', required: false }];
test('it meets the interface', () => {
    let c = Command;
    expect(c).toBeDefined();
});
test('shows the ID', () => {
    expect(Command.id).toEqual('foo:bar');
});
test('runs the command', async () => {
    const { cmd, stdout } = await Command.mock();
    expect(cmd.flags).toEqual({});
    expect(cmd.argv).toEqual([]);
    expect(stdout).toEqual('foo\n');
});
test('has stdout', async () => {
    class Command extends command_1.Command {
        async run() {
            cli_ux_1.cli.stdout.log(this.flags.print);
        }
    }
    Command.flags = { print: cli_flags_1.flags.string(), bool: cli_flags_1.flags.boolean() };
    const { stdout } = await Command.mock('--print=foo');
    expect(stdout).toEqual('foo\n');
});
test('has stderr', async () => {
    class Command extends command_1.Command {
        async run() {
            cli_ux_1.cli.stderr.log(this.flags.print);
        }
    }
    Command.flags = { print: cli_flags_1.flags.string() };
    const { stderr } = await Command.mock('--print=foo');
    expect(stderr).toEqual('foo\n');
});
test('parses args', async () => {
    const { cmd } = await Command.mock('one');
    expect(cmd.flags).toEqual({});
    expect(cmd.argv).toEqual(['one']);
    expect(cmd.args).toEqual({ myarg: 'one' });
});
test('has help', async () => {
    class Command extends command_1.Command {
    }
    Command.topic = 'config';
    Command.command = 'get';
    Command.help = `this is

some multiline help
`;
    let config = cli_engine_config_1.buildConfig({ mock: true });
    expect(Command.buildHelp(config)).toEqual(`Usage: cli-engine config:get

this is

some multiline help\n`);
    expect(Command.buildHelpLine(config)).toEqual(['config:get', undefined]);
});
describe('http', () => {
    let api;
    let command;
    beforeEach(() => {
        api = nock('https://api.heroku.com');
        nock.disableNetConnect();
        command = new Command();
    });
    afterEach(() => {
        api.done();
    });
    test('makes an HTTP request', async () => {
        api = nock('https://api.heroku.com', {
            reqheaders: {
                'user-agent': `cli-engine/0.0.0 (darwin-x64) node-${process.version}`,
            },
        });
        api.get('/').reply(200, { message: 'ok' });
        let command = new Command({ config: { platform: 'darwin', arch: 'x64' } });
        let { body } = await command.http.get('https://api.heroku.com');
        expect(body).toEqual({ message: 'ok' });
    });
    describe('.post', async () => {
        test('makes a post request with body', async () => {
            api.post('/', { karate: 'chop', judo: 'throw', jujitsu: 'strangle' }).reply(200, { message: 'ok' });
            const body = { karate: 'chop', judo: 'throw', jujitsu: 'strangle' };
            await command.http.post('https://api.heroku.com', { body: body });
        });
    });
    test('stream', async () => {
        api = nock('https://api.heroku.com', {
            reqheaders: {
                'user-agent': `cli-engine/0.0.0 (darwin-x64) node-${process.version}`,
            },
        });
        api.get('/').reply(200, { message: 'ok' });
        let command = new Command({ config: { platform: 'darwin', arch: 'x64' } });
        const { response } = await command.http.stream('https://api.heroku.com');
        const body = JSON.parse(await concat(response));
        expect(body).toEqual({ message: 'ok' });
    });
    function concat(stream) {
        return new Promise(resolve => {
            let strings = [];
            stream.on('data', data => strings.push(data));
            stream.on('end', () => resolve(strings.join('')));
        });
    }
});
